@@ -10,6 +10,7 @@ import sys
 import re
 
 modules = [
+    "2ch",
     "2chan",
     "2chen",
     "35photo",
@@ -24,9 +25,11 @@ modules = [
     "architizer",
     "artstation",
     "aryion",
+    "batoto",
     "bbc",
     "behance",
     "blogger",
+    "bluesky",
     "bunkr",
     "catbox",
     "chevereto",
@@ -52,7 +55,7 @@ modules = [
     "gelbooru_v01",
     "gelbooru_v02",
     "gofile",
-    "hbrowse",
+    "hatenablog",
     "hentai2read",
     "hentaicosplays",
     "hentaifoundry",
@@ -107,7 +110,6 @@ modules = [
     "nitter",
     "nozomi",
     "nsfwalbum",
-    "nudecollect",
     "paheal",
     "patreon",
     "philomena",
@@ -122,8 +124,10 @@ modules = [
     "pixnet",
     "plurk",
     "poipiku",
+    "poringa",
     "pornhub",
     "pornpics",
+    "postmill",
     "pururin",
     "reactor",
     "readcomiconline",
@@ -143,6 +147,7 @@ modules = [
     "smugmug",
     "soundgasm",
     "speakerdeck",
+    "steamgriddb",
     "subscribestar",
     "szurubooru",
     "tapas",
@@ -173,9 +178,11 @@ modules = [
     "weibo",
     "wikiart",
     "wikifeet",
+    "wikimedia",
     "xhamster",
     "xvideos",
     "zerochan",
+    "zzup",
     "booru",
     "moebooru",
     "foolfuuka",
