@@ -38,7 +38,7 @@ class DeviantartExtractor(Extractor):
 
     def __init__(self, match):
         Extractor.__init__(self, match)
-        self.user = match.group(1) or match.group(2)
+        self.user = (match.group(1) or match.group(2) or "").lower()
         self.offset = 0
 
     def _init(self):
@@ -48,6 +48,7 @@ class DeviantartExtractor(Extractor):
         self.quality = self.config("quality", "100")
         self.original = self.config("original", True)
         self.comments = self.config("comments", False)
+        self.intermediary = self.config("intermediary", True)
 
         self.api = DeviantartOAuthAPI(self)
         self.group = False
@@ -61,7 +62,12 @@ class DeviantartExtractor(Extractor):
             self.unwatch = None
 
         if self.quality:
-            self.quality = ",q_{}".format(self.quality)
+            if self.quality == "png":
+                self.quality = "-fullview.png?"
+                self.quality_sub = re.compile(r"-fullview\.[a-z0-9]+\?").sub
+            else:
+                self.quality = ",q_{}".format(self.quality)
+                self.quality_sub = re.compile(r",q_\d+").sub
 
         if self.original != "image":
             self._update_content = self._update_content_default
@@ -103,7 +109,6 @@ class DeviantartExtractor(Extractor):
                     raise exception.StopExtraction()
                 else:
                     self.subcategory = "group-" + self.subcategory
-                    self.user = self.user.lower()
                     self.group = True
 
         for deviation in self.deviations():
@@ -119,6 +124,12 @@ class DeviantartExtractor(Extractor):
                     "Skipping %s (deleted)", deviation["deviationid"])
                 continue
 
+            tier_access = deviation.get("tier_access")
+            if tier_access == "locked":
+                self.log.debug(
+                    "Skipping %s (access locked)", deviation["deviationid"])
+                continue
+
             if "premium_folder_data" in deviation:
                 data = self._fetch_premium(deviation)
                 if not data:
@@ -129,25 +140,7 @@ class DeviantartExtractor(Extractor):
             yield Message.Directory, deviation
 
             if "content" in deviation:
-                content = deviation["content"]
-
-                if self.original and deviation["is_downloadable"]:
-                    self._update_content(deviation, content)
-                elif self.jwt:
-                    self._update_token(deviation, content)
-                elif content["src"].startswith("https://images-wixmp-"):
-                    if deviation["index"] <= 790677560:
-                        # https://github.com/r888888888/danbooru/issues/4069
-                        intermediary, count = re.subn(
-                            r"(/f/[^/]+/[^/]+)/v\d+/.*",
-                            r"/intermediary\1", content["src"], 1)
-                        if count:
-                            deviation["_fallback"] = (content["src"],)
-                            content["src"] = intermediary
-                    if self.quality:
-                        content["src"] = re.sub(
-                            r",q_\d+", self.quality, content["src"], 1)
-
+                content = self._extract_content(deviation)
                 yield self.commit(deviation, content)
 
             elif deviation["is_downloadable"]:
@@ -332,6 +325,33 @@ class DeviantartExtractor(Extractor):
         deviation["extension"] = "txt"
         return Message.Url, txt, deviation
 
+    def _extract_content(self, deviation):
+        content = deviation["content"]
+
+        if self.original and deviation["is_downloadable"]:
+            self._update_content(deviation, content)
+            return content
+
+        if self.jwt:
+            self._update_token(deviation, content)
+            return content
+
+        if content["src"].startswith("https://images-wixmp-"):
+            if self.intermediary and deviation["index"] <= 790677560:
+                # https://github.com/r888888888/danbooru/issues/4069
+                intermediary, count = re.subn(
+                    r"(/f/[^/]+/[^/]+)/v\d+/.*",
+                    r"/intermediary\1", content["src"], 1)
+                if count:
+                    deviation["is_original"] = False
+                    deviation["_fallback"] = (content["src"],)
+                    content["src"] = intermediary
+            if self.quality:
+                content["src"] = self.quality_sub(
+                    self.quality, content["src"], 1)
+
+        return content
+
     @staticmethod
     def _find_folder(folders, name, uuid):
         if uuid.isdecimal():
@@ -451,9 +471,11 @@ class DeviantartExtractor(Extractor):
             return None
 
         dev = self.api.deviation(deviation["deviationid"], False)
-        folder = dev["premium_folder_data"]
+        folder = deviation["premium_folder_data"]
         username = dev["author"]["username"]
-        has_access = folder["has_access"]
+
+        # premium_folder_data is no longer present when user has access (#5063)
+        has_access = ("premium_folder_data" not in dev) or folder["has_access"]
 
         if not has_access and folder["type"] == "watchers" and \
                 self.config("auto-watch"):
@@ -511,11 +533,13 @@ class DeviantartUserExtractor(DeviantartExtractor):
     def items(self):
         base = "{}/{}/".format(self.root, self.user)
         return self._dispatch_extractors((
-            (DeviantartGalleryExtractor , base + "gallery"),
-            (DeviantartScrapsExtractor  , base + "gallery/scraps"),
-            (DeviantartJournalExtractor , base + "posts"),
-            (DeviantartStatusExtractor  , base + "posts/statuses"),
-            (DeviantartFavoriteExtractor, base + "favourites"),
+            (DeviantartAvatarExtractor    , base + "avatar"),
+            (DeviantartBackgroundExtractor, base + "banner"),
+            (DeviantartGalleryExtractor   , base + "gallery"),
+            (DeviantartScrapsExtractor    , base + "gallery/scraps"),
+            (DeviantartJournalExtractor   , base + "posts"),
+            (DeviantartStatusExtractor    , base + "posts/statuses"),
+            (DeviantartFavoriteExtractor  , base + "favourites"),
         ), ("gallery",))
 
 
@@ -536,6 +560,70 @@ class DeviantartGalleryExtractor(DeviantartExtractor):
         return self._folder_urls(folders, "gallery", DeviantartFolderExtractor)
 
 
+class DeviantartAvatarExtractor(DeviantartExtractor):
+    """Extractor for an artist's avatar"""
+    subcategory = "avatar"
+    archive_fmt = "a_{_username}_{index}"
+    pattern = BASE_PATTERN + r"/avatar"
+    example = "https://www.deviantart.com/USER/avatar/"
+
+    def deviations(self):
+        name = self.user.lower()
+        profile = self.api.user_profile(name)
+        if not profile:
+            return ()
+
+        user = profile["user"]
+        icon = user["usericon"]
+        index = icon.rpartition("?")[2]
+
+        formats = self.config("formats")
+        if not formats:
+            url = icon.replace("/avatars/", "/avatars-big/", 1)
+            return (self._make_deviation(url, user, index, ""),)
+
+        if isinstance(formats, str):
+            formats = formats.replace(" ", "").split(",")
+
+        results = []
+        for fmt in formats:
+            fmt, _, ext = fmt.rpartition(".")
+            if fmt:
+                fmt = "-" + fmt
+            url = "https://a.deviantart.net/avatars{}/{}/{}/{}.{}?{}".format(
+                fmt, name[0], name[1], name, ext, index)
+            results.append(self._make_deviation(url, user, index, fmt))
+        return results
+
+    def _make_deviation(self, url, user, index, fmt):
+        return {
+            "author"         : user,
+            "category"       : "avatar",
+            "index"          : text.parse_int(index),
+            "is_deleted"     : False,
+            "is_downloadable": False,
+            "published_time" : 0,
+            "title"          : "avatar" + fmt,
+            "stats"          : {"comments": 0},
+            "content"        : {"src": url},
+        }
+
+
+class DeviantartBackgroundExtractor(DeviantartExtractor):
+    """Extractor for an artist's banner"""
+    subcategory = "background"
+    archive_fmt = "b_{index}"
+    pattern = BASE_PATTERN + r"/ba(?:nner|ckground)"
+    example = "https://www.deviantart.com/USER/banner/"
+
+    def deviations(self):
+        try:
+            return (self.api.user_profile(self.user.lower())
+                    ["cover_deviation"]["cover_deviation"],)
+        except Exception:
+            return ()
+
+
 class DeviantartFolderExtractor(DeviantartExtractor):
     """Extractor for deviations inside an artist's gallery folder"""
     subcategory = "folder"
@@ -1003,8 +1091,9 @@ class DeviantartOAuthAPI():
         self.strategy = extractor.config("pagination")
         self.public = extractor.config("public", True)
 
-        self.client_id = extractor.config("client-id")
-        if self.client_id:
+        client_id = extractor.config("client-id")
+        if client_id:
+            self.client_id = str(client_id)
             self.client_secret = extractor.config("client-secret")
         else:
             self.client_id = self.CLIENT_ID
@@ -1012,7 +1101,7 @@ class DeviantartOAuthAPI():
 
         token = extractor.config("refresh-token")
         if token is None or token == "cache":
-            token = "#" + str(self.client_id)
+            token = "#" + self.client_id
             if not _refresh_token_cache(token):
                 token = None
         self.refresh_token_key = token
@@ -1578,7 +1667,7 @@ class DeviantartEclipseAPI():
         return token
 
 
-@cache(maxage=100*365*86400, keyarg=0)
+@cache(maxage=36500*86400, keyarg=0)
 def _refresh_token_cache(token):
     if token and token[0] == "#":
         return None
