@@ -1,33 +1,40 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2019 Mike Fährmann
+# Copyright 2017-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extract images from https://www.flickr.com/"""
+"""Extractors for https://www.flickr.com/"""
 
 from .common import Extractor, Message
 from .. import text, oauth, util, exception
 
+BASE_PATTERN = r"(?:https?://)?(?:www\.|secure\.|m\.)?flickr\.com"
+
 
 class FlickrExtractor(Extractor):
     """Base class for flickr extractors"""
     category = "flickr"
     filename_fmt = "{category}_{id}.{extension}"
+    directory_fmt = ("{category}", "{user[username]}")
+    archive_fmt = "{id}"
+    cookies_domain = None
+    request_interval = (1.0, 2.0)
+    request_interval_min = 0.2
 
     def __init__(self, match):
         Extractor.__init__(self, match)
-        self.api = FlickrAPI(self)
         self.item_id = match.group(1)
+
+    def _init(self):
+        self.api = FlickrAPI(self)
         self.user = None
 
     def items(self):
         data = self.metadata()
         extract = self.api._extract_format
-        yield Message.Version, 1
-        yield Message.Directory, data
         for photo in self.photos():
             try:
                 photo = extract(photo)
@@ -38,6 +45,7 @@ class FlickrExtractor(Extractor):
             else:
                 photo.update(data)
                 url = photo["url"]
+                yield Message.Directory, photo
                 yield Message.Url, url, text.nameext_from_url(url, photo)
 
     def metadata(self):
@@ -52,42 +60,11 @@ class FlickrExtractor(Extractor):
 class FlickrImageExtractor(FlickrExtractor):
     """Extractor for individual images from flickr.com"""
     subcategory = "image"
-    archive_fmt = "{id}"
     pattern = (r"(?:https?://)?(?:"
-               r"(?:(?:www\.|m\.)?flickr\.com/photos/[^/]+/"
-               r"|[^.]+\.static\.?flickr\.com/(?:\d+/)+)(\d+)"
+               r"(?:(?:www\.|secure\.|m\.)?flickr\.com/photos/[^/?#]+/"
+               r"|[\w-]+\.static\.?flickr\.com/(?:\d+/)+)(\d+)"
                r"|flic\.kr/p/([A-Za-z1-9]+))")
-    test = (
-        ("https://www.flickr.com/photos/departingyyz/16089302239", {
-            "pattern": pattern,
-            "content": "0821a28ee46386e85b02b67cf2720063440a228c",
-            "keyword": {
-                "comments": int,
-                "description": str,
-                "extension": "jpg",
-                "filename": "16089302239_de18cd8017_b",
-                "id": 16089302239,
-                "height": 683,
-                "label": "Large",
-                "media": "photo",
-                "url": str,
-                "views": int,
-                "width": 1024,
-            },
-        }),
-        ("https://www.flickr.com/photos/145617051@N08/46733161535", {
-            "count": 1,
-            "keyword": {"media": "video"},
-        }),
-        ("http://c2.staticflickr.com/2/1475/24531000464_9a7503ae68_b.jpg", {
-            "pattern": pattern}),
-        ("https://farm2.static.flickr.com/1035/1188352415_cb139831d0.jpg", {
-            "pattern": pattern}),
-        ("https://flic.kr/p/FPVo9U", {
-            "pattern": pattern}),
-        ("https://www.flickr.com/photos/zzz/16089302238", {
-            "exception": exception.NotFoundError}),
-    )
+    example = "https://www.flickr.com/photos/USER/12345"
 
     def __init__(self, match):
         FlickrExtractor.__init__(self, match)
@@ -98,12 +75,15 @@ class FlickrImageExtractor(FlickrExtractor):
 
     def items(self):
         photo = self.api.photos_getInfo(self.item_id)
+        if self.api.exif:
+            photo.update(self.api.photos_getExif(self.item_id))
 
         if photo["media"] == "video" and self.api.videos:
             self.api._extract_video(photo)
         else:
             self.api._extract_photo(photo)
 
+        photo["user"] = photo["owner"]
         photo["title"] = photo["title"]["_content"]
         photo["comments"] = text.parse_int(photo["comments"]["_content"])
         photo["description"] = photo["description"]["_content"]
@@ -119,7 +99,6 @@ class FlickrImageExtractor(FlickrExtractor):
                     location[key] = value["_content"]
 
         url = photo["url"]
-        yield Message.Version, 1
         yield Message.Directory, photo
         yield Message.Url, url, text.nameext_from_url(url, photo)
 
@@ -127,21 +106,11 @@ class FlickrImageExtractor(FlickrExtractor):
 class FlickrAlbumExtractor(FlickrExtractor):
     """Extractor for photo albums from flickr.com"""
     subcategory = "album"
-    directory_fmt = ("{category}", "{subcategory}s",
-                     "{album[id]} - {album[title]}")
+    directory_fmt = ("{category}", "{user[username]}",
+                     "Albums", "{album[id]} {album[title]}")
     archive_fmt = "a_{album[id]}_{id}"
-    pattern = (r"(?:https?://)?(?:www\.)?flickr\.com/"
-               r"photos/([^/]+)/(?:album|set)s(?:/(\d+))?")
-    test = (
-        (("https://www.flickr.com/photos/shona_s/albums/72157633471741607"), {
-            "pattern": FlickrImageExtractor.pattern,
-            "count": 6,
-        }),
-        ("https://www.flickr.com/photos/shona_s/albums", {
-            "pattern": pattern,
-            "count": 2,
-        }),
-    )
+    pattern = BASE_PATTERN + r"/photos/([^/?#]+)/(?:album|set)s(?:/(\d+))?"
+    example = "https://www.flickr.com/photos/USER/albums/12345"
 
     def __init__(self, match):
         FlickrExtractor.__init__(self, match)
@@ -153,7 +122,6 @@ class FlickrAlbumExtractor(FlickrExtractor):
         return self._album_items()
 
     def _album_items(self):
-        yield Message.Version, 1
         data = FlickrExtractor.metadata(self)
         data["_extractor"] = FlickrAlbumExtractor
 
@@ -176,16 +144,11 @@ class FlickrAlbumExtractor(FlickrExtractor):
 class FlickrGalleryExtractor(FlickrExtractor):
     """Extractor for photo galleries from flickr.com"""
     subcategory = "gallery"
-    directory_fmt = ("{category}", "galleries",
-                     "{user[username]} {gallery[id]}")
+    directory_fmt = ("{category}", "{user[username]}",
+                     "Galleries", "{gallery[gallery_id]} {gallery[title]}")
     archive_fmt = "g_{gallery[id]}_{id}"
-    pattern = (r"(?:https?://)?(?:www\.)?flickr\.com/"
-               r"photos/([^/]+)/galleries/(\d+)")
-    test = (("https://www.flickr.com/photos/flickr/"
-             "galleries/72157681572514792/"), {
-        "pattern": FlickrImageExtractor.pattern,
-        "count": ">= 10",
-    })
+    pattern = BASE_PATTERN + r"/photos/([^/?#]+)/galleries/(\d+)"
+    example = "https://www.flickr.com/photos/USER/galleries/12345/"
 
     def __init__(self, match):
         FlickrExtractor.__init__(self, match)
@@ -203,13 +166,10 @@ class FlickrGalleryExtractor(FlickrExtractor):
 class FlickrGroupExtractor(FlickrExtractor):
     """Extractor for group pools from flickr.com"""
     subcategory = "group"
-    directory_fmt = ("{category}", "{subcategory}s", "{group[groupname]}")
+    directory_fmt = ("{category}", "Groups", "{group[groupname]}")
     archive_fmt = "G_{group[nsid]}_{id}"
-    pattern = r"(?:https?://)?(?:www\.)?flickr\.com/groups/([^/]+)"
-    test = ("https://www.flickr.com/groups/bird_headshots/", {
-        "pattern": FlickrImageExtractor.pattern,
-        "count": "> 150",
-    })
+    pattern = BASE_PATTERN + r"/groups/([^/?#]+)"
+    example = "https://www.flickr.com/groups/NAME/"
 
     def metadata(self):
         self.group = self.api.urls_lookupGroup(self.item_id)
@@ -222,13 +182,9 @@ class FlickrGroupExtractor(FlickrExtractor):
 class FlickrUserExtractor(FlickrExtractor):
     """Extractor for the photostream of a flickr user"""
     subcategory = "user"
-    directory_fmt = ("{category}", "{user[username]}")
     archive_fmt = "u_{user[nsid]}_{id}"
-    pattern = r"(?:https?://)?(?:www\.)?flickr\.com/photos/([^/]+)/?$"
-    test = ("https://www.flickr.com/photos/shona_s/", {
-        "pattern": FlickrImageExtractor.pattern,
-        "count": 28,
-    })
+    pattern = BASE_PATTERN + r"/photos/([^/?#]+)/?$"
+    example = "https://www.flickr.com/photos/USER/"
 
     def photos(self):
         return self.api.people_getPhotos(self.user["nsid"])
@@ -237,13 +193,10 @@ class FlickrUserExtractor(FlickrExtractor):
 class FlickrFavoriteExtractor(FlickrExtractor):
     """Extractor for favorite photos of a flickr user"""
     subcategory = "favorite"
-    directory_fmt = ("{category}", "{subcategory}s", "{user[username]}")
+    directory_fmt = ("{category}", "{user[username]}", "Favorites")
     archive_fmt = "f_{user[nsid]}_{id}"
-    pattern = r"(?:https?://)?(?:www\.)?flickr\.com/photos/([^/]+)/favorites"
-    test = ("https://www.flickr.com/photos/shona_s/favorites", {
-        "pattern": FlickrImageExtractor.pattern,
-        "count": 4,
-    })
+    pattern = BASE_PATTERN + r"/photos/([^/?#]+)/favorites"
+    example = "https://www.flickr.com/photos/USER/favorites"
 
     def photos(self):
         return self.api.favorites_getList(self.user["nsid"])
@@ -252,14 +205,10 @@ class FlickrFavoriteExtractor(FlickrExtractor):
 class FlickrSearchExtractor(FlickrExtractor):
     """Extractor for flickr photos based on search results"""
     subcategory = "search"
-    directory_fmt = ("{category}", "{subcategory}", "{search[text]}")
+    directory_fmt = ("{category}", "Search", "{search[text]}")
     archive_fmt = "s_{search}_{id}"
-    pattern = r"(?:https?://)?(?:www\.)?flickr\.com/search/?\?([^#]+)"
-    test = (
-        ("https://flickr.com/search/?text=mountain"),
-        ("https://flickr.com/search/?text=tree%20cloud%20house"
-         "&color_codes=4&styles=minimalism"),
-    )
+    pattern = BASE_PATTERN + r"/search/?\?([^#]+)"
+    example = "https://flickr.com/search/?text=QUERY"
 
     def __init__(self, match):
         FlickrExtractor.__init__(self, match)
@@ -275,23 +224,31 @@ class FlickrSearchExtractor(FlickrExtractor):
 
 
 class FlickrAPI(oauth.OAuth1API):
-    """Minimal interface for the flickr API"""
+    """Minimal interface for the flickr API
+
+    https://www.flickr.com/services/api/
+    """
+
     API_URL = "https://api.flickr.com/services/rest/"
-    API_KEY = "ac4fd7aa98585b9eee1ba761c209de68"
-    API_SECRET = "3adb0f568dc68393"
+    API_KEY = "f8f78d1a40debf471f0b22fa2d00525f"
+    API_SECRET = "4f9dae1113e45556"
     FORMATS = [
-        ("o", "Original"    , None),
-        ("k", "Large 2048"  , 2048),
-        ("h", "Large 1600"  , 1600),
-        ("l", "Large"       , 1024),
-        ("c", "Medium 800"  , 800),
-        ("z", "Medium 640"  , 640),
-        ("m", "Medium"      , 500),
-        ("n", "Small 320"   , 320),
-        ("s", "Small"       , 240),
-        ("q", "Large Square", 150),
-        ("t", "Thumbnail"   , 100),
-        ("s", "Square"      , 75),
+        ("o" , "Original"    , None),
+        ("6k", "X-Large 6K"  , 6144),
+        ("5k", "X-Large 5K"  , 5120),
+        ("4k", "X-Large 4K"  , 4096),
+        ("3k", "X-Large 3K"  , 3072),
+        ("k" , "Large 2048"  , 2048),
+        ("h" , "Large 1600"  , 1600),
+        ("l" , "Large"       , 1024),
+        ("c" , "Medium 800"  , 800),
+        ("z" , "Medium 640"  , 640),
+        ("m" , "Medium"      , 500),
+        ("n" , "Small 320"   , 320),
+        ("s" , "Small"       , 240),
+        ("q" , "Large Square", 150),
+        ("t" , "Thumbnail"   , 100),
+        ("s" , "Square"      , 75),
     ]
     VIDEO_FORMATS = {
         "orig"       : 9,
@@ -309,6 +266,7 @@ class FlickrAPI(oauth.OAuth1API):
     def __init__(self, extractor):
         oauth.OAuth1API.__init__(self, extractor)
 
+        self.exif = extractor.config("exif", False)
         self.videos = extractor.config("videos", True)
         self.maxsize = extractor.config("size-max")
         if isinstance(self.maxsize, str):
@@ -325,7 +283,7 @@ class FlickrAPI(oauth.OAuth1API):
                             if not fmt[2] or fmt[2] <= self.maxsize]
         else:
             self.formats = self.FORMATS
-        self.formats = self.formats[:4]
+        self.formats = self.formats[:8]
 
     def favorites_getList(self, user_id):
         """Returns a list of the user's favorite photos."""
@@ -353,6 +311,11 @@ class FlickrAPI(oauth.OAuth1API):
         params = {"user_id": user_id}
         return self._pagination("people.getPhotos", params)
 
+    def photos_getExif(self, photo_id):
+        """Retrieves a list of EXIF/TIFF/GPS tags for a given photo."""
+        params = {"photo_id": photo_id}
+        return self._call("photos.getExif", params)["photo"]
+
     def photos_getInfo(self, photo_id):
         """Get information about a photo."""
         params = {"photo_id": photo_id}
@@ -402,9 +365,11 @@ class FlickrAPI(oauth.OAuth1API):
         """Returns a user NSID, given the url to a user's photos or profile."""
         params = {"url": "https://www.flickr.com/photos/" + username}
         user = self._call("urls.lookupUser", params)["user"]
-        return {"nsid": user["id"],
-                "path_alias": username,
-                "username": user["username"]["_content"]}
+        return {
+            "nsid"      : user["id"],
+            "username"  : user["username"]["_content"],
+            "path_alias": username,
+        }
 
     def video_getStreamInfo(self, video_id, secret=None):
         """Returns all available video streams"""
@@ -421,7 +386,11 @@ class FlickrAPI(oauth.OAuth1API):
         params["nojsoncallback"] = "1"
         if self.api_key:
             params["api_key"] = self.api_key
-        data = self.request(self.API_URL, params=params).json()
+        response = self.request(self.API_URL, params=params)
+        try:
+            data = response.json()
+        except ValueError:
+            data = {"code": -1, "message": response.content}
         if "code" in data:
             msg = data.get("message")
             self.log.debug("Server response: %s", data)
@@ -435,8 +404,19 @@ class FlickrAPI(oauth.OAuth1API):
         return data
 
     def _pagination(self, method, params, key="photos"):
-        params["extras"] = "description,date_upload,tags,views,media,"
-        params["extras"] += ",".join("url_" + fmt[0] for fmt in self.formats)
+        extras = ("description,date_upload,tags,views,media,"
+                  "path_alias,owner_name,")
+        includes = self.extractor.config("metadata")
+        if includes:
+            if isinstance(includes, (list, tuple)):
+                includes = ",".join(includes)
+            elif not isinstance(includes, str):
+                includes = ("license,date_taken,original_format,last_update,"
+                            "geo,machine_tags,o_dims")
+            extras = extras + includes + ","
+        extras += ",".join("url_" + fmt[0] for fmt in self.formats)
+
+        params["extras"] = extras
         params["page"] = 1
 
         while True:
@@ -461,8 +441,22 @@ class FlickrAPI(oauth.OAuth1API):
         photo["views"] = text.parse_int(photo["views"])
         photo["date"] = text.parse_timestamp(photo["dateupload"])
         photo["tags"] = photo["tags"].split()
+
+        if self.exif:
+            photo.update(self.photos_getExif(photo["id"]))
         photo["id"] = text.parse_int(photo["id"])
 
+        if "owner" in photo:
+            photo["owner"] = {
+                "nsid"      : photo["owner"],
+                "username"  : photo["ownername"],
+                "path_alias": photo["pathalias"],
+            }
+        else:
+            photo["owner"] = self.extractor.user
+        del photo["pathalias"]
+        del photo["ownername"]
+
         if photo["media"] == "video" and self.videos:
             return self._extract_video(photo)
 
