@@ -25,7 +25,7 @@ class IdolcomplexExtractor(SankakuExtractor):
     cookies_domain = "idol.sankakucomplex.com"
     cookies_names = ("_idolcomplex_session",)
     referer = False
-    request_interval = (4.0, 6.0)
+    request_interval = (3.0, 6.0)
 
     def __init__(self, match):
         SankakuExtractor.__init__(self, match)
@@ -34,8 +34,11 @@ class IdolcomplexExtractor(SankakuExtractor):
         self.start_post = 0
 
     def _init(self):
+        self.find_pids = re.compile(
+            r" href=[\"#]/\w\w/posts/(\w+)"
+        ).findall
         self.find_tags = re.compile(
-            r'tag-type-([^"]+)">\s*<div [^>]+>\s*<a href="/\?tags=([^"]+)'
+            r'tag-type-([^"]+)">\s*<a [^>]*?href="/[^?]*\?tags=([^"]+)'
         ).findall
 
     def items(self):
@@ -67,7 +70,7 @@ class IdolcomplexExtractor(SankakuExtractor):
 
         self.logged_in = False
 
-    @cache(maxage=90*24*3600, keyarg=1)
+    @cache(maxage=90*86400, keyarg=1)
     def _login_impl(self, username, password):
         self.log.info("Logging in as %s", username)
 
@@ -98,7 +101,7 @@ class IdolcomplexExtractor(SankakuExtractor):
         page = self.request(url, retries=10).text
         extr = text.extract_from(page)
 
-        tags = extr("<title>", " | ")
+        pid_alnum = extr('/posts/', '"')
         vavg = extr('itemprop="ratingValue">', "<")
         vcnt = extr('itemprop="reviewCount">', "<")
         pid = extr(">Post ID:", "<")
@@ -118,8 +121,8 @@ class IdolcomplexExtractor(SankakuExtractor):
 
         data = {
             "id"          : text.parse_int(pid),
+            "id_alnum"    : pid_alnum,
             "md5"         : file_url.rpartition("/")[2].partition(".")[0],
-            "tags"        : text.unescape(tags),
             "vote_average": text.parse_float(vavg),
             "vote_count"  : text.parse_int(vcnt),
             "created_at"  : created,
@@ -149,8 +152,8 @@ class IdolcomplexTagExtractor(IdolcomplexExtractor):
     subcategory = "tag"
     directory_fmt = ("{category}", "{search_tags}")
     archive_fmt = "t_{search_tags}_{id}"
-    pattern = r"(?:https?://)?idol\.sankakucomplex\.com/\?([^#]*)"
-    example = "https://idol.sankakucomplex.com/?tags=TAGS"
+    pattern = BASE_PATTERN + r"/(?:posts/?)?\?([^#]*)"
+    example = "https://idol.sankakucomplex.com/en/posts?tags=TAGS"
     per_page = 20
 
     def __init__(self, match):
@@ -196,14 +199,15 @@ class IdolcomplexTagExtractor(IdolcomplexExtractor):
             page = self.request(self.root, params=params, retries=10).text
             pos = ((page.find('id="more-popular-posts-link"') + 1) or
                    (page.find('<span class="thumb') + 1))
-            yield from text.extract_iter(page, ' href="/posts/', '"', pos)
+
+            yield from self.find_pids(page, pos)
 
             next_url = text.extract(page, 'next-page-url="', '"', pos)[0]
             if not next_url:
                 return
 
-            next_params = text.parse_query(text.unescape(text.unescape(
-                next_url).lstrip("?/")))
+            next_params = text.parse_query(text.unquote(text.unescape(
+                text.unescape(next_url).lstrip("?/"))))
 
             if "next" in next_params:
                 # stop if the same "next" value occurs twice in a row (#265)
@@ -218,7 +222,7 @@ class IdolcomplexPoolExtractor(IdolcomplexExtractor):
     subcategory = "pool"
     directory_fmt = ("{category}", "pool", "{pool}")
     archive_fmt = "p_{pool}_{id}"
-    pattern = r"(?:https?://)?idol\.sankakucomplex\.com/pools?/show/(\d+)"
+    pattern = BASE_PATTERN + r"/pools?/show/(\d+)"
     example = "https://idol.sankakucomplex.com/pools/show/12345"
     per_page = 24
 
@@ -242,8 +246,7 @@ class IdolcomplexPoolExtractor(IdolcomplexExtractor):
         while True:
             page = self.request(url, params=params, retries=10).text
             pos = page.find('id="pool-show"') + 1
-            post_ids = list(text.extract_iter(
-                page, ' href="/posts/', '"', pos))
+            post_ids = self.find_pids(page, pos)
 
             yield from post_ids
             if len(post_ids) < self.per_page:
@@ -255,7 +258,7 @@ class IdolcomplexPostExtractor(IdolcomplexExtractor):
     """Extractor for single images from idol.sankakucomplex.com"""
     subcategory = "post"
     archive_fmt = "{id}"
-    pattern = BASE_PATTERN + r"/posts?/(?:show/)?([0-9a-f]+)"
+    pattern = BASE_PATTERN + r"/posts?/(?:show/)?(\w+)"
     example = "https://idol.sankakucomplex.com/posts/0123456789abcdef"
 
     def __init__(self, match):
