@@ -22,22 +22,34 @@ class SkebExtractor(Extractor):
     def __init__(self, match):
         Extractor.__init__(self, match)
         self.user_name = match.group(1)
+
+    def _init(self):
         self.thumbnails = self.config("thumbnails", False)
         self.article = self.config("article", False)
 
     def items(self):
+        metadata = self.metadata()
         for user_name, post_num in self.posts():
             response, post = self._get_post_data(user_name, post_num)
+            if metadata:
+                post.update(metadata)
+
+            files = self._get_files_from_post(response)
+            post["count"] = len(files)
             yield Message.Directory, post
-            for data in self._get_urls_from_post(response, post):
-                url = data["file_url"]
-                yield Message.Url, url, text.nameext_from_url(url, data)
+            for post["num"], file in enumerate(files, 1):
+                post.update(file)
+                url = file["file_url"]
+                yield Message.Url, url, text.nameext_from_url(url, post)
 
     def posts(self):
         """Return post number"""
 
+    def metadata(self):
+        """Return additional metadata"""
+
     def _pagination(self, url, params):
-        headers = {"Referer": self.root, "Authorization": "Bearer null"}
+        headers = {"Authorization": "Bearer null"}
         params["offset"] = 0
 
         while True:
@@ -61,7 +73,7 @@ class SkebExtractor(Extractor):
     def _get_post_data(self, user_name, post_num):
         url = "{}/api/users/{}/works/{}".format(
             self.root, user_name, post_num)
-        headers = {"Referer": self.root, "Authorization": "Bearer null"}
+        headers = {"Authorization": "Bearer null"}
         resp = self.request(url, headers=headers).json()
         creator = resp["creator"]
         post = {
@@ -71,9 +83,6 @@ class SkebExtractor(Extractor):
             "body"             : resp["body"],
             "source_body"      : resp["source_body"],
             "translated_body"  : resp["translated"],
-            "completed_at"     : resp["completed_at"],
-            "date"             : text.parse_datetime(
-                resp["completed_at"], "%Y-%m-%dT%H:%M:%S.%fZ"),
             "nsfw"             : resp["nsfw"],
             "anonymous"        : resp["anonymous"],
             "tags"             : resp["tag_list"],
@@ -100,95 +109,55 @@ class SkebExtractor(Extractor):
             }
         return resp, post
 
-    def _get_urls_from_post(self, resp, post):
+    def _get_files_from_post(self, resp):
+        files = []
+
         if self.thumbnails and "og_image_url" in resp:
-            post["content_category"] = "thumb"
-            post["file_id"] = "thumb"
-            post["_file_id"] = str(resp["id"]) + "t"
-            post["file_url"] = resp["og_image_url"]
-            yield post
+            files.append({
+                "content_category": "thumb",
+                "file_id" : "thumb",
+                "_file_id": str(resp["id"]) + "t",
+                "file_url": resp["og_image_url"],
+            })
 
         if self.article and "article_image_url" in resp:
             url = resp["article_image_url"]
             if url:
-                post["content_category"] = "article"
-                post["file_id"] = "article"
-                post["_file_id"] = str(resp["id"]) + "a"
-                post["file_url"] = url
-                yield post
+                files.append({
+                    "content_category": "article",
+                    "file_id" : "article",
+                    "_file_id": str(resp["id"]) + "a",
+                    "file_url": url,
+                })
 
         for preview in resp["previews"]:
-            post["content_category"] = "preview"
-            post["file_id"] = post["_file_id"] = preview["id"]
-            post["file_url"] = preview["url"]
             info = preview["information"]
-            post["original"] = {
-                "width"     : info["width"],
-                "height"    : info["height"],
-                "byte_size" : info["byte_size"],
-                "duration"  : info["duration"],
-                "frame_rate": info["frame_rate"],
-                "software"  : info["software"],
-                "extension" : info["extension"],
-                "is_movie"  : info["is_movie"],
-                "transcoder": info["transcoder"],
-            }
-            yield post
+            files.append({
+                "content_category": "preview",
+                "file_id" : preview["id"],
+                "_file_id": preview["id"],
+                "file_url": preview["url"],
+                "original": {
+                    "width"     : info["width"],
+                    "height"    : info["height"],
+                    "byte_size" : info["byte_size"],
+                    "duration"  : info["duration"],
+                    "frame_rate": info["frame_rate"],
+                    "software"  : info["software"],
+                    "extension" : info["extension"],
+                    "is_movie"  : info["is_movie"],
+                    "transcoder": info["transcoder"],
+                },
+            })
+
+        return files
 
 
 class SkebPostExtractor(SkebExtractor):
     """Extractor for a single skeb post"""
     subcategory = "post"
     pattern = r"(?:https?://)?skeb\.jp/@([^/?#]+)/works/(\d+)"
-    test = ("https://skeb.jp/@kanade_cocotte/works/38", {
-        "count": 2,
-        "keyword": {
-            "anonymous": False,
-            "body": "re:はじめまして。私はYouTubeにてVTuberとして活動をしている湊ラ",
-            "client": {
-                "avatar_url": r"re:https://pbs.twimg.com/profile_images"
-                              r"/\d+/\w+\.jpg",
-                "header_url": r"re:https://pbs.twimg.com/profile_banners"
-                              r"/1375007870291300358/\d+/1500x500",
-                "id": 1196514,
-                "name": str,
-                "screen_name": "minato_ragi",
-            },
-            "completed_at": "2022-02-27T14:03:45.442Z",
-            "content_category": "preview",
-            "creator": {
-                "avatar_url": "https://pbs.twimg.com/profile_images"
-                              "/1225470417063645184/P8_SiB0V.jpg",
-                "header_url": "https://pbs.twimg.com/profile_banners"
-                              "/71243217/1647958329/1500x500",
-                "id": 159273,
-                "name": "イチノセ奏",
-                "screen_name": "kanade_cocotte",
-            },
-            "date": "dt:2022-02-27 14:03:45",
-            "file_id": int,
-            "file_url": str,
-            "genre": "art",
-            "nsfw": False,
-            "original": {
-                "byte_size": int,
-                "duration": None,
-                "extension": "re:psd|png",
-                "frame_rate": None,
-                "height": 3727,
-                "is_movie": False,
-                "width": 2810,
-            },
-            "post_num": "38",
-            "post_url": "https://skeb.jp/@kanade_cocotte/works/38",
-            "source_body": None,
-            "source_thanks": None,
-            "tags": list,
-            "thanks": None,
-            "translated_body": False,
-            "translated_thanks": None,
-        }
-    })
+    example = "https://skeb.jp/@USER/works/123"
 
     def __init__(self, match):
         SkebExtractor.__init__(self, match)
@@ -202,13 +171,7 @@ class SkebUserExtractor(SkebExtractor):
     """Extractor for all posts from a skeb user"""
     subcategory = "user"
     pattern = r"(?:https?://)?skeb\.jp/@([^/?#]+)/?$"
-    test = ("https://skeb.jp/@kanade_cocotte", {
-        "pattern": r"https://skeb\.imgix\.net/uploads/origins/[\w-]+"
-                   r"\?bg=%23fff&auto=format&txtfont=bold&txtshad=70"
-                   r"&txtclr=BFFFFFFF&txtalign=middle%2Ccenter&txtsize=150"
-                   r"&txt=SAMPLE&w=800&s=\w+",
-        "range": "1-5",
-    })
+    example = "https://skeb.jp/@USER"
 
     def posts(self):
         url = "{}/api/users/{}/works".format(self.root, self.user_name)
@@ -223,11 +186,63 @@ class SkebUserExtractor(SkebExtractor):
         return posts
 
 
+class SkebSearchExtractor(SkebExtractor):
+    """Extractor for skeb search results"""
+    subcategory = "search"
+    pattern = r"(?:https?://)?skeb\.jp/search\?q=([^&#]+)"
+    example = "https://skeb.jp/search?q=QUERY"
+
+    def metadata(self):
+        return {"search_tags": text.unquote(self.user_name)}
+
+    def posts(self):
+        url = "https://hb1jt3kre9-2.algolianet.com/1/indexes/*/queries"
+        params = {
+            "x-algolia-agent": "Algolia for JavaScript (4.13.1); Browser",
+        }
+        headers = {
+            "Origin": self.root,
+            "x-algolia-api-key": "9a4ce7d609e71bf29e977925e4c6740c",
+            "x-algolia-application-id": "HB1JT3KRE9",
+        }
+
+        filters = self.config("filters")
+        if filters is None:
+            filters = ("genre:art OR genre:voice OR genre:novel OR "
+                       "genre:video OR genre:music OR genre:correction")
+        elif not isinstance(filters, str):
+            filters = " OR ".join(filters)
+
+        page = 0
+        pams = "hitsPerPage=40&filters=" + text.quote(filters) + "&page="
+
+        request = {
+            "indexName": "Request",
+            "query": text.unquote(self.user_name),
+            "params": pams + str(page),
+        }
+        data = {"requests": (request,)}
+
+        while True:
+            result = self.request(
+                url, method="POST", params=params, headers=headers, json=data,
+            ).json()["results"][0]
+
+            for post in result["hits"]:
+                parts = post["path"].split("/")
+                yield parts[1][1:], parts[3]
+
+            if page >= result["nbPages"]:
+                return
+            page += 1
+            request["params"] = pams + str(page)
+
+
 class SkebFollowingExtractor(SkebExtractor):
     """Extractor for all creators followed by a skeb user"""
     subcategory = "following"
     pattern = r"(?:https?://)?skeb\.jp/@([^/?#]+)/following_creators"
-    test = ("https://skeb.jp/@user/following_creators",)
+    example = "https://skeb.jp/@USER/following_creators"
 
     def items(self):
         for user in self.users():
@@ -238,8 +253,8 @@ class SkebFollowingExtractor(SkebExtractor):
     def users(self):
         url = "{}/api/users/{}/following_creators".format(
             self.root, self.user_name)
-        headers = {"Referer": self.root, "Authorization": "Bearer null"}
         params = {"sort": "date", "offset": 0, "limit": 90}
+        headers = {"Authorization": "Bearer null"}
 
         while True:
             data = self.request(url, params=params, headers=headers).json()
